import click
import numpy as np
import pandas as pd

from prebio_df import read_prebio

def missing_values_table(df: pd.DataFrame, title:str=None) -> pd.DataFrame:
    """Generate a table of missing values in the dataframe.
    
    Args:
        df (pd.DataFrame): Input dataframe.
        title (str, optional): Title for the missing values table. Defaults to '_____ NAN ANALYSIS {df.shape} _____'.
    
    Returns:
        pd.DataFrame: Table of missing values with columns 'Missing Values' and '% of Total Values'.
    """
    mis_val = df.isnull().sum()
    mis_val_percent = 100 * mis_val / len(df)
    mis_percent = 100 * sum(mis_val.values) / np.prod(df.shape)
    mis_val_table = pd.concat([mis_val, mis_val_percent], axis=1)
    
    new_columns = {0 : 'Missing Values', 1 : '% of Total Values'}
    mis_val_table_ren_columns = mis_val_table.rename(columns = new_columns)
    
    mis_val_table_ren_columns = mis_val_table_ren_columns[mis_val_table_ren_columns.iloc[:,1] != 0] \
        .sort_values('% of Total Values', ascending=False) \
        .round(1)
        
    msg = f'_____ NAN ANALYSIS {df.shape} _____'
    print(f'{msg if title is None else title}')
    print(f'> Your selected dataframe has {df.shape[1]} columns and {df.shape[0]} samples.')
    print(f'> A {mis_percent: .4}% of the dataset is missing')
    columns_with_nans = mis_val_table_ren_columns.where(mis_val_table_ren_columns['% of Total Values'] > 0).index
    print(f'> There are {len(columns_with_nans)} columns with missing values.')
    
    if not mis_val_table_ren_columns.empty:
        numerical_cols = df[columns_with_nans].select_dtypes(include=np.number)
        categorical_cols = df[columns_with_nans].select_dtypes(include=object)
        print("> From those columns:")
        print(f"  > {len(categorical_cols.columns)} columns are categorical")
        print(f"  > {len(numerical_cols.columns)} columns are numerical")
    
    return mis_val_table_ren_columns

def get_nans_per_row(df) -> pd.DataFrame:
    """ Count number of NaNs per row and add a column

    Returns:
        pd.DataFrame: Dataframe with the column of number of NaNs added
    """
    data = df

    cols = list(data.columns)
    cols.append('NaNs')

    new_data = pd.DataFrame(columns=cols)

    for index in data.index:
        row = data.iloc[index, :]
        nans = row.isnull().sum()
        new_row = list(row)
        new_row.append(nans)
        new_data.loc[len(new_data)] = new_row
        
    return new_data


def generate_images(df):
    import seaborn as sns
    import seaborn_image as snsi
    import matplotlib.pyplot as plt
    
    res = df.isnull()
    snsi.imgplot(res, gray=True)
    plt.tight_layout()
    plt.savefig('NaN.png')
    plt.close()
    
    res = missing_values_table(df)
    res_len = res.shape[0]
    res.reset_index(inplace=True)
    res = res.rename(columns = {'index': f'Feature ID ({res_len} columns)'})
    res = res.drop(columns=['% of Total Values'])
    
    sns.barplot(res, x= f'Feature ID ({res_len} columns)', y='Missing Values')
    sns.despine()
    plt.tick_params(
        axis='x',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom=False,      # ticks along the bottom edge are off
        top=False,         # ticks along the top edge are off
        labelbottom=False
    )
    
    plt.tight_layout()
    plt.savefig('NaNPerColumn.png',  transparent=True)
    plt.close()
    